/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#pragma once

#include <QObject>
#include <QGenericMatrix>

#include "remotecontrol.qpb.h"
#include "remotecontrol_grpc.qpb.h"

#include "valueindicator.h"
#include "abstractdense.h"
#include "layertrigger.h"

class ValueIndicator;
class LayerTrigger;

struct NetworkLayerState {
    ValueIndicatorDense m_activations;
    ValueIndicatorDense m_biases;
    ValueIndicatorDense m_weights;
    LayerTrigger m_actiovationTrigger;
    LayerTrigger m_weightTrigger;
};

class VisualizerModel : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QList<int> sizes READ sizes NOTIFY sizesChanged)
    Q_PROPERTY(remotecontrol::NetworkState* networkState READ networkState CONSTANT)
public:
    explicit VisualizerModel(std::shared_ptr<remotecontrol::RemoteControlClient> &client, QObject *parent = nullptr);

    QList<int> sizes() {
        return m_networkConfig.sizes();
    }

    Q_INVOKABLE ValueIndicator *activation(int layer, int row);
    Q_INVOKABLE ValueIndicator *weight(int layer, int row, int column);
    Q_INVOKABLE LayerTrigger *activationTrigger(int layer);
    Q_INVOKABLE LayerTrigger *weightTrigger(int layer);
    Q_INVOKABLE void start();

    remotecontrol::NetworkState *networkState() const
    {
        return m_networkState.data();
    }

signals:
    void sizesChanged();

private:
    std::shared_ptr<remotecontrol::RemoteControlClient> &m_client;
    remotecontrol::Configuration m_networkConfig;
    QList<NetworkLayerState*> m_layers;
    QPointer<remotecontrol::NetworkState> m_networkState;
};
