/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef HANDWRITINGENGINE_H
#define HANDWRITINGENGINE_H

#include <QObject>
#include <QUrl>
#include <QByteArray>
#include <QtProtobufTypes>

#include "handwriting.qpb.h"

#include <memory>

namespace handwriting {
    class HandwritingClient;
}
class HandwritingEngine final : public QObject
{
    Q_OBJECT
    Q_PROPERTY(int result READ result WRITE setResult NOTIFY resultChanged)
    Q_PROPERTY(handwriting::Matrix *matrix READ matrix CONSTANT)
public:
    explicit HandwritingEngine(QObject *parent = nullptr);
    virtual ~HandwritingEngine() = default;

    Q_INVOKABLE void retrain();
    Q_INVOKABLE void recognize();
    Q_INVOKABLE void setNeuralNetworkData(const QString &networkDataPath);
    Q_INVOKABLE void updateValue(int index, double value);

    int result() const
    {
        return m_result;
    }

    handwriting::Matrix *matrix()
    {
        return &m_matrix;
    }

public slots:
    void setResult(int result)
    {
        if (m_result == result)
            return;

        m_result = result;
        emit resultChanged(m_result);
    }

signals:
    void resultChanged(int result);

private:
    std::shared_ptr<handwriting::HandwritingClient> m_client;
    int m_result;
    QtProtobuf::DoubleList m_data;
    handwriting::Matrix m_matrix;
};

#endif // HANDWRITINGENGINE_H
