/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>
#include <QDebug>
#include <QtProtobufTypes>

#include <QGrpcHttp2Channel>
#include <QGrpcInsecureCredentials>
#include "qtprotobuf_global.qpb.h"
#include "snakesimulator_grpc.qpb.h"

#include "clientwrapper.h"

int main(int argc, char *argv[])
{
    QGuiApplication app(argc, argv);
    QtProtobuf::qRegisterProtobufTypes();
    snakesimulator::qRegisterProtobufTypes();

    qmlRegisterUncreatableType<QtProtobuf::QGrpcAsyncReply>("snakesimulator", 1, 0, "QGrpcAsyncReply", "");
    std::shared_ptr<snakesimulator::SnakeSimulatorClient> client(new snakesimulator::SnakeSimulatorClient);
    auto chan = std::shared_ptr<QtProtobuf::QGrpcHttp2Channel>(new QtProtobuf::QGrpcHttp2Channel(QUrl("http://localhost:65002"), QtProtobuf::QGrpcInsecureCallCredentials()|QtProtobuf::QGrpcInsecureChannelCredentials()));
    client->attachChannel(chan);

    ClientWrapper *wrap = new ClientWrapper(client.get());

    snakesimulator::Snake *snake = new snakesimulator::Snake;
    QPointer<snakesimulator::Field> field = new snakesimulator::Field;
    QPointer<snakesimulator::Stats> stats = new snakesimulator::Stats;
    QPointer<snakesimulator::PlayingBestState> isPlaying = new snakesimulator::PlayingBestState;

    client->subscribeFieldUpdates({}, field);
    client->subscribeStatsUpdates({}, stats);
    client->subscribeIsPlayingUpdates({}, isPlaying);

    auto subscription = client->subscribeSnakeUpdates({});

    QObject::connect(subscription, &QtProtobuf::QGrpcSubscription::updated, [subscription, snake](){
        snake->setPoints(subscription->read<snakesimulator::Snake>().points()); //Issue https://github.com/semlanik/qtprotobuf/issues/48
    });

    QQmlApplicationEngine engine;
    engine.rootContext()->setContextProperty("field", field);
    engine.rootContext()->setContextProperty("snake", snake);
    engine.rootContext()->setContextProperty("stats", stats);
    engine.rootContext()->setContextProperty("client", wrap);
    engine.rootContext()->setContextProperty("isPlaying", isPlaying);

    engine.load(QUrl(QStringLiteral("qrc:/main.qml")));
    if (engine.rootObjects().isEmpty())
        return -1;

    return app.exec();
}
